@@ -1,13 +1,15 @@
 #!/usr/bin/env python3
 
 import os
-import time
+import threading
 
+from datetime import datetime
 from inspect import currentframe
 from configator.utils.trackback_util import CodeLocation
 from configator.utils.function_util import dict_update
 
 EMPTY_DICT = dict()
+HISTORY_LENGTH = 10
 
 class EnvHelper(CodeLocation):
     #
@@ -16,21 +18,33 @@ class EnvHelper(CodeLocation):
     __prefix = None
     __strict = None
     __tracking_enabled = None
+    __thread_safe = None
+    __lock = None
     __footprint = None
     #
     def __init__(self, *args, **kwargs):
         self.__prefix = None
         self.__strict = True
         self.__tracking_enabled = True
+        self.__thread_safe = False
+        self.__lock = threading.Lock()
         self.__footprint = dict()
     #
     def _track_env(self, key, value, info=None):
         if not self.__tracking_enabled:
             return self
         #
-        record = dict(value=value, time=time.time(), info=info)
+        if not self.__thread_safe:
+            return self.__track_env_unsafe_threading(key=key, value=value, info=info)
+        #
+        with self.__lock:
+            return self.__track_env_unsafe_threading(key=key, value=value, info=info)
+    #
+    def __track_env_unsafe_threading(self, key, value, info=None):
+        record = dict(value=value, time=datetime.utcnow(), info=info)
         if self.__footprint is None:
             self.__footprint = dict()
+        #
         if key not in self.__footprint:
             self.__footprint[key] = dict(
                 count=1,
@@ -42,6 +56,10 @@ class EnvHelper(CodeLocation):
             store['count'] = store['count'] + 1
             store['history'].append(store['current'])
             store['current'] = record
+            history = store['history']
+            while len(history) > HISTORY_LENGTH:
+                history.pop(0)
+        #
         return self
     #
     @property
@@ -59,6 +77,7 @@ class EnvHelper(CodeLocation):
             dict(
                 prefix=self.prefix,
                 strict=self.strict,
+                thread_safe=self.thread_safe,
                 values=self.__extract_values()
             ),
             dict(trails=trails), trails is not None)
@@ -78,6 +97,7 @@ class EnvHelper(CodeLocation):
             return dict(trail)
         return { k: extract(v) for k, v in self.__footprint.items() }
     #
+    #
     @property
     def prefix(self):
         self.__freeze = True
@@ -95,6 +115,7 @@ class EnvHelper(CodeLocation):
                 self.__prefix = self.__prefix.upper()
         return val
     #
+    #
     @property
     def strict(self):
         return self.__strict
@@ -105,6 +126,7 @@ class EnvHelper(CodeLocation):
             self.__strict = val
         return val
     #
+    #
     @property
     def tracking_enabled(self):
         return self.__tracking_enabled
@@ -116,6 +138,17 @@ class EnvHelper(CodeLocation):
         return val
     #
     #
+    @property
+    def thread_safe(self):
+        return self.__thread_safe
+    #
+    @thread_safe.setter
+    def thread_safe(self, val):
+        if isinstance(val, bool):
+            self.__thread_safe = val
+        return val
+    #
+    #
     def getenv(self, label, default=None, with_prefix=True):
         used_key = label
         prefixed_key = None
